/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.se.rmi.registry;

import java.net.MalformedURLException;
import java.rmi.AccessException;
import java.rmi.AlreadyBoundException;
import java.rmi.NoSuchObjectException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;

import org.objectweb.petals.tools.rmi.server.remote.interfaces.RemoteComponentContext;

/**
 * A class to simplify the RMI register creation.
 * @author Nicolas Salatge - EBM WebSourcing
 */
public class Registry {

    /**
     * The registry itself.
     */
    private static java.rmi.registry.Registry REGISTRY;

    /**
     * Rmi component context name.
     */
    private String rmiComponentContextName = null;

    /**
     * The registry host.
     */
    private String registryHost;

    /**
     * The registry port number.
     */
    private int registryPortNumber = -1;

    /**
     * Do we need to create a new registry.
     */
    private boolean createRegistry = true;

    /**
     * Creates a new instance of Registry.
     * 
     * @param properties
     *            the RMI properties
     */
    public Registry(final RmiProperties properties) {
        this.setRegistryHost(properties.getRmiHost());
        this.setRegistryPortNumber(properties.getRmiPort());
        this.setCreateRegistry(properties.isEmbeddedRegistry());
        this.setRmiComponentContextName(properties.getComponentContextName());
    }

    /**
     * Get the registry port number.
     * 
     * @return the port
     */
    public int getRegistryPortNumber() {
        return this.registryPortNumber;
    }

    /**
     * Set the registry port number.
     * 
     * @param v
     *            the port
     */
    public void setRegistryPortNumber(final int v) {
        this.registryPortNumber = v;
    }

    public String getRegistryHost() {
        return registryHost;
    }

    public void setRegistryHost(String registryHost) {
        this.registryHost = registryHost;
    }

    /**
     * 
     * 
     * @return {@code true} if the server registry is embedded to the component
     */
    public boolean isRegistryEmbedded() {
        return this.createRegistry;
    }

    /**
     * Set createRegistry value.
     * 
     * @param v
     *            the ShouldCreateRegistry
     */
    public void setCreateRegistry(final boolean create) {
        this.createRegistry = create;
    }

    /**
     * Get the RMI component context name.
     * 
     * @return the RMI component context name
     */
    public String getRmiComponentContextName() {
        return this.rmiComponentContextName;
    }

    /**
     * 
     * @param n
     *            the name of component context name
     */
    public void setRmiComponentContextName(final String n) {
        this.rmiComponentContextName = n;
    }

    /**
     * Initialize the registry. If the registry already exists, get the
     * references on it, else launch it...
     * 
     * @throws RemoteException
     *             Impossible to realize a remote access
     * 
     */
    public synchronized void initializeRegistry() throws RemoteException {
        if (this.createRegistry) {
            this.createAndGetRegistry();
        } else {
            REGISTRY = java.rmi.registry.LocateRegistry.getRegistry(this.registryHost,
                    this.registryPortNumber);
        }
    }

    /**
     * Detect the registry.
     * 
     * @param port
     *            the port
     */
    private void detectRegistry() {
        try {
            REGISTRY = java.rmi.registry.LocateRegistry.getRegistry(this.registryPortNumber);

            if (REGISTRY != null) {
                // call the registry ping service
                REGISTRY.lookup("ping");
            }
        } catch (java.rmi.NotBoundException e) {
        } catch (java.rmi.RemoteException e) {
            REGISTRY = null;
        }
    }

    /**
     * create and get the registry. Ignore the creation of the registry runs
     * already.
     * 
     * @param port
     *            the port
     * @return the registry
     * @throws java.rmi.RemoteException
     *             impossible to realize a remote access
     */
    private void createAndGetRegistry() throws java.rmi.RemoteException {
        // test if registry is already on
        this.detectRegistry();

        if (REGISTRY == null) {
            // no registry created
            try {
                REGISTRY = java.rmi.registry.LocateRegistry.createRegistry(this.registryPortNumber);
            } catch (java.rmi.RemoteException e) {
                // The registry may have been created by another component
                // during this creation
                this.detectRegistry();
                if (REGISTRY == null) {
                    throw e;
                }
            }
        }
    }

    /**
     * Method registerRMIInterface.
     * 
     * @param comp
     *            the component context
     * @throws RemoteException
     *             impossible to register this object
     * @throws MalformedURLException
     *             impossible to register this object
     * @throws AlreadyBoundException
     *             impossible to register this object
     */
    public void registerRMIInterface(final RemoteComponentContext comp) throws RemoteException,
            MalformedURLException, AlreadyBoundException {
        REGISTRY.bind(this.getRmiComponentContextName(), comp);
    }

    /**
     * Method deregisterRMIInterface.
     * 
     * @throws RemoteException
     * @throws RemoteException
     *             impossible to register this object
     * @throws MalformedURLException
     *             impossible to register this object
     * @throws NotBoundException
     *             impossible to register this object
     */
    public void deregisterRMIInterface() throws RemoteException, MalformedURLException,
            NotBoundException {
        REGISTRY.unbind(this.getRmiComponentContextName());
    }

    /**
     * Allow to shutdown RMI registry and unexport
     * all linked remote objects.
     * @param comp
     *            the component context
     * @throws <NotBoundException> 
     * @throws <RemoteException> 
     * @throws <AccessException>
     * @throws <NoSuchObjectException>
     */
    public void shutDown(final RemoteComponentContext comp) throws AccessException, RemoteException, NotBoundException {
        
        try {
        // Shutdown by unexport exportable object implementation
        java.rmi.server.UnicastRemoteObject.unexportObject(comp, true);
        } catch (NoSuchObjectException nsoe) {
            System.err.println("one more HACK in se-rmi!!");
        }
        try {
        // Shutdown by unexport concrete registry
        java.rmi.server.UnicastRemoteObject.unexportObject(REGISTRY, true);
        } catch (NoSuchObjectException nsoe) {
            System.err.println("one more HACK in se-rmi!!");
        }
    }
}
